@@ -3,7 +3,7 @@
 # fileio.R
 #
 # copyright (c) 2004, Carter T. Butts <buttsc@uci.edu>
-# Last Modified 4/15/06
+# Last Modified 5/2/09
 # Licensed under the GNU General Public License version 2 (June, 1991)
 #
 # Part of the R/sna package
@@ -23,7 +23,7 @@
 #by Matthijs den Besten.
 read.dot <- function(...) {
   lines <- readLines(...);
-  body <- lines[grep("->", lines, fixed=T)];
+  body <- lines[grep("->", lines, fixed=TRUE)];
 
   nodePairs <- sub('^[[:space:]]+\"', '\"',
                    sub('\"[;[:space:]]+$', '\"',
@@ -49,7 +49,7 @@ read.dot <- function(...) {
 
 #read.nos - Read an input file in Neo-OrgStat format.  At this time, only the 
 #graph stack is read; any coloring information is ignored.
-read.nos<-function(file){
+read.nos<-function(file,return.as.edgelist=FALSE){
    #Get the formatting information
    f<-sapply(readLines(file,n=2),strsplit," ")
    #Parse the formatting information
@@ -65,7 +65,10 @@ read.nos<-function(file){
          for(k in 1:o)
             gstack[i,j,k]<-dat[(i-1)*n*o+(j-1)*o+k]   
    #Return the stack
-   gstack
+   if(return.as.edgelist)
+     as.edgelist.sna(gstack)
+   else
+     gstack
 }
 
 
